def build_headers(token):
	headers = {'Content-Type': 'application/json', 'Authorization': f'Bearer {token}'}
	return headers


def build_pat_headers(site):
	headers = {'Content-Type': 'application/json', 'Authorization': f'pat {site.get("pat")}'}
	return headers



from requests import get


# def import_instances(instances):
# 	headers = build_headers(login())
# 	for instance in instances:
# 		response = post('http://localhost:8000/topic/data', data=dumps(instance), headers=headers)
# 		print(response.status_code)
#
#
# def remove_topic_collection(collections):
# 	headers = build_headers(login())
# 	response = post('http://localhost:8000/topic/data/remove', data=dumps(collections), headers=headers)
# 	print(response.status_code)


def test_url(url):
	response = get(url)
	return response



from base64 import b64decode
from json import loads
from os import walk

from watchmen_cli.meta import import_asset, import_md_asset
from watchmen_cli.utils import ArrayUtils


def list_asset_markdown(folder, markdown_file=None):
	_, _, filenames = next(walk(folder))
	asset = []
	for file_name in filenames:
		if markdown_file is not None and file_name == markdown_file:
			with open(f'{folder}/{file_name}', encoding='utf-8') as f:
				data = f.read()
				asset.append(data)
		else:
			if file_name.endswith(".md"):
				with open(f'{folder}/{file_name}', encoding='utf-8') as f:
					data = f.read()
					asset.append(data)
	return asset


def read_data_from_markdown(markdown: str):
	topic_list = []
	pipeline_list = []
	space_list = []
	connect_space_list = []

	array = ArrayUtils(markdown.split("\n"))
	json_list = array \
		.map(lambda x: x.strip()) \
		.filter(lambda x: x.startswith('<a href="data:application/json;base64,')) \
		.map(lambda x: x.replace('<a href="data:application/json;base64,', '')) \
		.map(lambda x: b64decode(x[0:x.find('"')])) \
		.map(lambda x: loads(x)) \
		.to_list()

	for json_data in json_list:
		if "pipelineId" in json_data:
			pipeline_list.append(json_data)
		elif "topicId" in json_data:
			topic_list.append(json_data)
		elif "spaceId" in json_data and "connectId" not in json_data:
			space_list.append(json_data)
		elif "connectId" in json_data:
			connect_space_list.append(json_data)
	return topic_list, pipeline_list, space_list, connect_space_list


# noinspection PyUnusedLocal
def import_markdowns(folder, site, import_type, markdown_file):
	markdown_list = list_asset_markdown(folder)
	for markdown in markdown_list:
		topic_list, pipeline_list, space_list, connect_space_list = read_data_from_markdown(markdown)
		import_asset(site, {
			'topics': topic_list,
			'pipelines': pipeline_list,
			'spaces': space_list,
			'connectedSpaces': connect_space_list,
			'importType': import_type
		})


def import_markdowns_v2(host, token, import_type):
	markdown_list = list_asset_markdown_v2('config')
	for markdown in markdown_list:
		topic_list, pipeline_list, space_list, connect_space_list = read_data_from_markdown(markdown)
		data_ = {
			'topics': topic_list,
			'pipelines': pipeline_list,
			'spaces': space_list,
			'connectedSpaces': connect_space_list,
			'importType': import_type
		}
		import_md_asset(host, token, data_)


def list_asset_markdown_v2(folder):
	_, _, filenames = next(walk(folder))
	asset = []
	for file_name in filenames:
		if file_name.endswith('.md'):
			with open(f'{folder}/{file_name}', encoding='utf-8') as f:
				data = f.read()
				asset.append(data)
	return asset



from requests import get

from .auth import login
from .utils import build_headers


def search_template_space(site, space_id):
	headers = build_headers(login(site))
	response = get(f'{site.get("host")}console_space/template/list?space_id={space_id}', headers=headers)
	connect_template_list = response.json()
	template_space_list = []
	for connect_template in connect_template_list:
		connect_id = connect_template["connectId"]
		response = get(f'{site.get("host")}console_space?connect_id={connect_id}', headers=headers)
		template_space_list.append(response.json())
	return template_space_list


# def load


# noinspection PyUnusedLocal
def search_template_chart(site, name):
	pass


# noinspection PyUnusedLocal
def search_template_subject(site, name):
	pass


# noinspection PyUnusedLocal
def load_template_space(site, space_id: str):
	pass


# noinspection PyUnusedLocal
def import_template_space(site, template_space):
	pass


# noinspection PyUnusedLocal
def load_template_subject(site, subject_id):
	pass


# noinspection PyUnusedLocal
def import_template_subject(site, template_subject):
	pass


# noinspection PyUnusedLocal
def load_template_chart(site, chart_id):
	pass


# noinspection PyUnusedLocal
def import_template_chart(site, template_chart):
	pass



import functools


class ArrayUtils(object):
	def __init__(self, data_list):
		self.data = data_list

	def map(self, lambda_):
		self.data = list(map(lambda_, self.data))
		return self

	def filter(self, lambda_):
		self.data = list(filter(lambda_, self.data))
		return self

	def to_list(self):
		return self.data

	def reduce(self, lambda_, initial_):
		return functools.reduce(lambda_, self.data, initial_)



from requests import post


def login(site):
	login_data = {'username': site.get('username'), 'password': site.get('password')}
	headers = {'Content-Type': 'application/x-www-form-urlencoded'}
	response = post(f'{site.get("host")}login/access-token', data=login_data, headers=headers)
	auth_token = response.json().get('access_token')
	return auth_token



from typing import List

from watchmen_cli.admin import search_spaces, search_topics, search_user_groups, search_users


def search_user(site, name):
	users = search_users(site, name)
	for user in users:
		print(f'User[id={user.get("userId")}, name={user.get("name")}].')


def search_topic(site, name):
	results: List = search_topics(site, name)
	for result in results:
		print(f'Topic[id={result.get("topicId")}, name={result.get("name")}].')
	return results


def search_space(site, name):
	results: List = search_spaces(site, name)
	for result in results:
		print(f'Space[id={result.get("spaceId")}, name={result.get("name")}].')
	return results


def search_user_group(site, name):
	user_groups = search_user_groups(site, name)
	for group in user_groups:
		print(f'User group[id={group.get("userGroupId")}, name={group.get("name")}].')


# noinspection PyUnusedLocal
def search_report(site, name):
	pass


# noinspection PyUnusedLocal
def search_connect_space(site, name):
	pass


# noinspection PyUnusedLocal
def search_subject(site, name):
	pass



from json import dumps

from requests import get, post

from .auth import login
from .utils import build_headers


def as_url(site, url: str) -> str:
	return f'{site.get("host")}{url}'


def search_topics(site, name):
	headers = build_headers(login(site))
	response = get(as_url(site, f'topic/name?query_name={name}'), headers=headers)
	return response.json()


def import_topics(site, topics):
	headers = build_headers(login(site))
	for topic in topics:
		response = post(as_url(site, 'topic/import'), data=dumps(topic), headers=headers)
		if response.status_code == 200:
			print("import successfully")
		else:
			print(response.text)


def load_topic_list(site, names):
	headers = build_headers(login(site))
	response = post(as_url(site, 'topic/list/name'), data=dumps(names), headers=headers)
	return response.json()


# noinspection PyUnusedLocal
def load_user_list(site, username_list):
	headers = build_headers(login(site))


def search_spaces(site, name):
	headers = build_headers(login(site))
	response = get(as_url(site, f'space/name?query_name={name}'), headers=headers)
	return response.json()


def load_space_list(site, names):
	headers = build_headers(login(site))
	response = post(as_url(site, 'space/list/name'), data=dumps(names), headers=headers)

	return response.json()


def load_user_groups(site, names):
	headers = build_headers(login(site))
	response = post(as_url(site, 'user_group/list/name'), data=dumps(names), headers=headers)

	data = response.json()
	# print(data)
	return data


def import_spaces(site, spaces):
	headers = build_headers(login(site))
	for space in spaces:
		response = post(as_url(site, 'space/import'), data=dumps(space), headers=headers)
		if response.status_code == 200:
			print("import successfully")


def list_all_pipeline(site):
	headers = build_headers(login(site))
	response = get(as_url(site, 'pipeline/all'), headers=headers)
	return response.json()


def load_pipeline_by_id(site, pipeline_id):
	headers = build_headers(login(site))
	response = get(as_url(site, f'pipeline?pipeline_id={pipeline_id}'), headers=headers)
	return response.json()


def import_pipelines(site, pipelines):
	headers = build_headers(login(site))
	for pipeline in pipelines:
		response = post(as_url(site, 'pipeline/import'), data=dumps(pipeline), headers=headers)
		if response.status_code == 200:
			print("import successfully")
		else:
			print(response.text)


def search_user_groups(site, name):
	headers = build_headers(login(site))
	response = get(as_url(site, f'user_group/name?query_name={name}'), headers=headers)
	return response.json()


def import_user_groups(site, groups):
	headers = build_headers(login(site))
	for group in groups:
		response = post(as_url(site, 'user_group/import'), data=dumps(group), headers=headers)
		if response.status_code == 200:
			print("import successfully")


def search_users(site, name):
	headers = build_headers(login(site))
	response = get(as_url(site, f'user/name?query_name={name}'), headers=headers)
	# print(response.status_code)
	return response.json()


def import_users(site, users):
	headers = build_headers(login(site))
	for user in users:
		response = post(as_url(site, 'user/import'), data=dumps(user), headers=headers)
		if response.status_code == 200:
			print("import successfully")


def load_users(site, names):
	print(dumps(names))
	headers = build_headers(login(site))
	response = post(as_url(site, 'user/list/name'), data=dumps(names), headers=headers)
	data = response.json()
	# print(response.json())
	# print(data)
	return data



from watchmen_cli.admin import import_pipelines, import_spaces, import_topics, import_user_groups, import_users, \
	list_all_pipeline, load_pipeline_by_id, load_space_list, load_topic_list, load_user_groups, load_users
from watchmen_cli.constants import FILE
from watchmen_cli.utils import load_from_file, save_to_file


def list_pipeline(site):
	pipeline_list = list_all_pipeline(site)
	for pipeline in pipeline_list:
		print(f'Pipeline[id={pipeline.get("pipelineId")}, name={pipeline.get("name")}].')


def sync_pipeline(source_site, target_site, ids):
	if source_site != FILE:
		pipeline_list = []
		for pipeline_id in ids:
			pipeline_list.append(load_pipeline_by_id(source_site, pipeline_id))
	else:
		pipeline_list: list = load_from_file(source_site, 'pipeline')

	print("find {} pipeline".format(len(pipeline_list)))

	if target_site != FILE:
		import_pipelines(target_site, pipeline_list)
	else:
		save_to_file(target_site, pipeline_list, 'pipeline')


def sync_user(source_site, target_site, names):
	if source_site != FILE:
		users: list = load_users(source_site, names)
	else:
		users: list = load_from_file(source_site, 'users')
	print(f'Found {len(users)} users.')
	if target_site != FILE:
		import_users(target_site, users)
	else:
		save_to_file(target_site, users, 'users')


def sync_user_group(source_site, target_site, names):
	if source_site != FILE:
		groups: list = load_user_groups(source_site, names)
	else:
		groups: list = load_from_file(source_site, 'user_groups')

	print(f'Found {len(groups)} user groups.')
	if target_site != FILE:
		import_user_groups(target_site, groups)
	else:
		save_to_file(target_site, groups, 'user_groups')


def sync_space(source_site, target_site, names):
	if source_site != FILE:
		space_list: list = load_space_list(source_site, names)
	else:
		space_list: list = load_from_file(source_site, 'space')

	print(f'Found {len(space_list)} spaces.')
	if target_site != FILE:
		import_spaces(target_site, space_list)
	else:
		save_to_file(target_site, space_list, 'space')


def sync_topic(source_site, target_site, names):
	if source_site != FILE:
		topic_list: list = load_topic_list(source_site, names)
	else:
		topic_list: list = load_from_file(source_site, 'topic')
	print(f'Found {len(topic_list)} topics.')
	if target_site != FILE:
		import_topics(target_site, topic_list)
	else:
		save_to_file(target_site, topic_list, 'topic')


# noinspection PyUnusedLocal
def __sync_connect_spaces(source_site, target_site, ids):
	pass


# noinspection PyUnusedLocal
def __sync_dashboards(source_site, target_site, ids):
	pass


# noinspection PyUnusedLocal
def __sync_subjects(source_site, target_site, ids):
	pass


# noinspection PyUnusedLocal
def __sync_reports(source_site, target_site, ids):
	pass



from json import dump, load
from os import makedirs, PathLike
from pathlib import Path
from typing import Any, List, Union


def __get_path(site_name: str, model_name: str):
	return f'./temp/{model_name}/{site_name}'


def save_to_file(site_name: str, data_list: List[Any], model_name: str) -> None:
	path = __get_path(site_name, model_name)
	makedirs(path, exist_ok=True)
	with open(f'{path}/{model_name}.json', 'w') as outfile:
		dump(data_list, outfile)


def load_from_file(site_name: str, model_name: str) -> Any:
	path = __get_path(site_name, model_name)
	with open(f'{path}/{model_name}.json', 'r') as outfile:
		return load(outfile)


def create_file(path: str, file_name: str, data_list: List[Any]) -> None:
	with open(f'{path}/{file_name}.json', 'w') as outfile:
		dump(data_list, outfile)


def load_file_to_json(file: Union[str, PathLike]) -> Any:
	with open(file, 'r') as outfile:
		return load(outfile)


def load_folder(folder_path: str) -> Path:
	return Path(folder_path)


def create_folder(folder_path: str) -> None:
	makedirs(folder_path, exist_ok=True)



from enum import Enum
from json import dump, dumps, load
from os import path
from typing import Any, Dict, List

from requests import post

from .common import test_url
from .constants import FILE
from .service import import_markdowns, import_markdowns_v2, list_pipeline, search_space, search_topic, search_user, \
	search_user_group, sync_pipeline, sync_space, sync_topic, sync_user, sync_user_group
from .utils import create_file, create_folder, load_file_to_json, load_folder

IMPORT = 'import'
GENERATE = 'generate'
COMBINE = 'combine'


class ModelType(str, Enum):
	TOPIC = 'topic'
	PIPELINE = 'pipeline'
	USER = 'user'
	USER_GROUP = 'user_group'
	SPACE = 'space'
	CONNECT_SPACE = 'connect_space'
	SUBJECT = 'subject'
	REPORT = 'report'
	DASHBOARD = 'dashboard'


def get_access_token(host: str, username: str, password: str) -> str:
	login_data = {'username': username, 'password': password, 'grant_type': 'password'}
	headers = {'Content-Type': 'application/x-www-form-urlencoded'}
	response = post(f'{host}/login/access-token', data=login_data, headers=headers)
	token = response.json().get('access_token')
	return token


# noinspection DuplicatedCode
def import_topics_to_env(token: str, host: str, topics: List[Dict[str, Any]]) -> None:
	headers = {'Content-Type': 'application/json', 'Authorization': 'Bearer ' + token}
	for topic in topics:
		response = post(
			f'{host}/topic/import', data=dumps(topic), headers=headers)
		if response.status_code == 200:
			print(f'Import topic[{topic.get("name")}] successfully')
		else:
			print(f'Import topic[{topic.get("name")}] failed.')


# noinspection DuplicatedCode
def import_pipelines_to_env(token: str, host: str, pipelines: List[Dict[str, Any]]) -> None:
	headers = {'Content-Type': 'application/json', 'Authorization': 'Bearer ' + token}
	for pipeline in pipelines:
		response = post(
			f'{host}/pipeline/import', data=dumps(pipeline), headers=headers)
		if response.status_code == 200:
			print(f'Import pipeline[{pipeline.get("name")}] successfully.')
		else:
			print(f'Import pipeline[{pipeline.get("name")}] failed.')


class CliBootstrap:
	# noinspection PyMethodMayBeStatic
	def __load_site_json(self):
		data = {}
		if path.exists('site/site.json'):
			with open('site/site.json', 'r') as outfile:
				data = load(outfile)
				return data
		else:
			return data

	# noinspection PyMethodMayBeStatic
	def __test_url(self, url: str) -> None:
		response = test_url(url)
		print(response.status_code)
		print(response.text)

	# noinspection PyMethodMayBeStatic
	def __save_to_json(self, data: Dict[str, Any]) -> None:
		with open('temp/site.json', 'w') as outfile:
			dump(data, outfile)

	def add_site(self, name: str, host: str, username: str = None, password: str = None) -> None:
		sites = self.__load_site_json()
		sites[name] = {'host': host, 'username': username, 'password': password}
		self.__save_to_json(sites)

	def search(self, model_type: str, site: str, name: str) -> None:
		model_type = ModelType(model_type)

		switcher_search = {
			ModelType.TOPIC.value: search_topic,
			ModelType.SPACE.value: search_space,
			ModelType.USER.value: search_user,
			ModelType.USER_GROUP.value: search_user_group
		}

		sites = self.__load_site_json()
		switcher_search.get(model_type.value)(sites[site], name)

	def asset(self, folder, site=None, import_type=None, markdown_file=None):
		sites = self.__load_site_json()
		import_markdowns(folder, sites[site], import_type, markdown_file)

	def test(self, url):
		self.__test_url(url)

	def hosts(self):
		print(self.__load_site_json())

	def list(self, model_type, site):
		model_type = ModelType(model_type)
		sites = self.__load_site_json()

		switcher_list = {
			ModelType.PIPELINE.value: list_pipeline
		}

		switcher_list.get(model_type)(sites[site])

	# noinspection PyDefaultArgument
	def sync(self, model_type: str, source: str, target: str, keys: List[str] = []) -> None:
		model_type = ModelType(model_type)
		sites = self.__load_site_json()
		switcher_sync = {
			ModelType.TOPIC.value: sync_topic,
			ModelType.SPACE.value: sync_space,
			ModelType.USER_GROUP.value: sync_user_group,
			ModelType.PIPELINE.value: sync_pipeline,
			ModelType.USER.value: sync_user

		}

		if target == FILE:
			target_site = FILE
		else:
			target_site = sites[target]

		if source == FILE:
			source_site = FILE
		else:
			source_site = sites[source]

		switcher_sync.get(model_type.value)(source_site, target_site, keys)

	# noinspection PyMethodMayBeStatic
	def raw(self, op_type, a_path: str):
		if op_type == COMBINE:
			root_folder = load_folder(a_path)
			for folder in root_folder.iterdir():
				if folder.is_dir():
					instance_path = str(folder.resolve()) + '_instance'
					instance_list = []
					create_folder(instance_path)
					for p in folder.iterdir():
						if p.is_file():
							instance_list.append(load_file_to_json(p))
					create_file(instance_path, f'{folder.name}-instance.json', instance_list)
			print('All instances are created')
		else:
			raise KeyError(f'Type[{op_type}] is not supported.')

	def verify_topic(self):
		pass

	def deploy(self, host: str, username: str, password: str):
		self.deploy_topics(host, username, password)
		self.deploy_pipelines(host, username, password)

	# noinspection PyMethodMayBeStatic
	def deploy_topics(self, host: str, username: str, password: str):
		try:
			token = get_access_token(host, username, password)
			print('import topics first')
			with open('/app/config/topic/topic.json', 'r') as src:
				topics = load(src)
				import_topics_to_env(token, host, topics)
		except Exception as err:
			raise err

	# noinspection PyMethodMayBeStatic
	def deploy_pipelines(self, host: str, username: str, password: str):
		try:
			token = get_access_token(host, username, password)
			print('import pipelines')
			with open('/app/config/pipeline/pipeline.json', 'r') as src:
				pipelines = load(src)
				import_pipelines_to_env(token, host, pipelines)
		except Exception as err:
			raise err

	def deploy_template(self, host: str, username: str, password: str):
		pass

	# noinspection PyMethodMayBeStatic
	def deploy_asset(self, host: str, username: str, password: str):
		try:
			token = get_access_token(host, username, password)
			print('Import markdown asset')
			print(token)
			import_markdowns_v2(host, token, 'replace')
		except Exception as err:
			raise err



from json import dumps

from requests import post

from .auth import login
from .utils import build_headers


def build_pat_headers(site):
	headers = {'Content-Type': 'application/json', 'Authorization': f'pat {site.get("pat")}'}
	return headers


def import_asset(site, import_asset_request):
	# headers =
	response = post(f'{site.get("host")}import', data=dumps(import_asset_request), headers=get_http_headers(site))
	if response.status_code == 200:
		print(response.json())
	else:
		print(response.text)


def get_http_headers(site):
	if 'pat' in site:
		headers = build_pat_headers(site)
	else:
		headers = build_headers(login(site))
	# access_token = login(site)
	return headers


def import_md_asset(host, token, data):
	headers = {'Content-Type': 'application/json', 'Authorization': 'Bearer ' + token}
	response = post(f'{host}/import', data=dumps(data), headers=headers)
	if response.status_code == 200:
		print(response.json())
	else:
		print(response.text)



from .data_search import search_space, search_topic, search_user, search_user_group
from .data_sync import list_pipeline, sync_pipeline, sync_space, sync_topic, sync_user, sync_user_group
from .markdown_service import import_markdowns, import_markdowns_v2
